package com.qmth.boot.data.upgrade.config;

import com.qmth.boot.data.upgrade.annotation.DataUpgradeVersion;
import com.qmth.boot.data.upgrade.model.BootAppInfo;
import com.qmth.boot.data.upgrade.service.DataInitService;
import com.qmth.boot.data.upgrade.service.DataUpgradeService;
import com.qmth.boot.data.upgrade.utils.ClassHelper;
import com.qmth.boot.data.upgrade.utils.VersionComparator;
import com.zaxxer.hikari.HikariDataSource;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.context.event.ApplicationPreparedEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;
import org.springframework.util.ClassUtils;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 数据升级监听类
 *
 * @author: Deason
 * @since: 2025/3/3
 */
public class DataUpgradeListener implements ApplicationListener<ApplicationPreparedEvent>, DataUpgradeConstants {

    private static final Logger log = LoggerFactory.getLogger(DataUpgradeListener.class);

    private DataUpgradeProperties properties;

    @Override
    public void onApplicationEvent(ApplicationPreparedEvent event) {
        try {
            properties = new DataUpgradeProperties(event.getApplicationContext().getEnvironment());
            if (!properties.getEnable()) {
                // 未启用时，不再进行后续处理
                return;
            }

            if (properties.getFirstInstall()) {
                // 首次安装时，仅执行数据初始化
                this.handlerDataInit();
            } else {
                // 否则执行数据升级
                this.handlerDataUpgrade();
            }
        } catch (Exception e) {
            log.error("数据操作异常终止！原因：{}", e.getMessage(), e);
            System.exit(1);
        }
    }

    public void handlerDataInit() {
        try (HikariDataSource dataSource = this.initDataSource()) {
            JdbcTemplate jdbcTemplate = new JdbcTemplate(dataSource);

            log.warn("待初始化应用码:{} 待初始化应用版本号:{}", properties.getAppCode(), properties.getAppVersion());
            if (this.existBootAppInfo(jdbcTemplate)) {
                throw new RuntimeException("【应用信息表】已存在，不允许重复数据初始化！");
            }

            List<DataInitService> services = ClassHelper.findInterfaceInstances(DataInitService.class, properties.getBasePackage());
            if (services.isEmpty()) {
                log.warn("未找到数据初始化的具体实现！{}:{}", BASE_PACKAGE, properties.getBasePackage());
                throw new RuntimeException("未找到数据初始化的具体实现！");
            }

            if (services.size() > 1) {
                throw new RuntimeException("不允许存在多个数据初始化的具体实现！");
            }

            log.warn("*************** data init start ***************");
            try {
                services.get(0).process(jdbcTemplate);
            } catch (Exception e) {
                throw new RuntimeException("SQL语句执行异常！", e);
            }

            // 数据初始化完成后，初始应用信息表
            this.initBootAppInfo(jdbcTemplate, properties.getAppCode(), properties.getAppVersion());
            log.warn("*************** data init finish ***************");
        } catch (Exception e) {
            throw new RuntimeException(e.getMessage(), e);
        }
    }

    public void handlerDataUpgrade() {
        try (HikariDataSource dataSource = this.initDataSource()) {
            JdbcTemplate jdbcTemplate = new JdbcTemplate(dataSource);
            TransactionTemplate transactionTemplate = new TransactionTemplate(new DataSourceTransactionManager(dataSource));

            if (!this.existBootAppInfo(jdbcTemplate)) {
                throw new RuntimeException("【应用信息表】不存在！");
            }

            BootAppInfo appInfo = this.queryBootAppInfo(jdbcTemplate, properties.getAppCode());
            if (appInfo == null) {
                throw new RuntimeException("【应用信息表】中未找到匹配的数据！appCode:" + properties.getAppCode());
            }

            log.warn("待升级应用码:{} 待升级应用版本号:{} 当前应用码:{} 当前应用版本号:{}", properties.getAppCode(),
                    properties.getAppVersion(), appInfo.getAppCode(), appInfo.getAppVersion());

            int compareValue = new VersionComparator().compare(appInfo.getAppVersion(), properties.getAppVersion());
            if (compareValue > 0) {
                throw new RuntimeException("待升级的版本号不能低于当前版本号！");
            }
            if (compareValue == 0) {
                log.warn("跳过数据升级，版本号一致！");
                return;
            }

            Map<String, DataUpgradeService> serviceMaps = this.matchServices(appInfo.getAppVersion(), properties.getAppVersion());
            if (serviceMaps.isEmpty()) {
                log.warn("未找到数据升级的具体实现！{}:{}", BASE_PACKAGE, properties.getBasePackage());
                throw new RuntimeException("未找到数据升级的具体实现！");
            }

            // 按版本号由小到大排序
            List<String> versionKeys = new ArrayList<>(serviceMaps.keySet());
            versionKeys.sort(new VersionComparator());

            transactionTemplate.execute(new TransactionCallbackWithoutResult() {
                @Override
                protected void doInTransactionWithoutResult(TransactionStatus status) {
                    try {
                        for (String version : versionKeys) {
                            log.warn("*************** data upgrade version:{} start ***************", version);
                            serviceMaps.get(version).process(jdbcTemplate);
                        }
                    } catch (Exception e) {
                        // DML-SQL语句支持事务‌，执行失败时触发自动回滚。
                        status.setRollbackOnly();
                        // DDL-SQL语句不支持事务‌，执行失败时请采用其它补偿机制回滚！！！
                        throw new RuntimeException("SQL语句执行异常！", e);
                    }
                }
            });

            // 数据升级完成后，更新应用信息表
            this.updateBootAppInfo(jdbcTemplate, properties.getAppCode(), properties.getAppVersion());
            log.warn("*************** data upgrade finish ***************");
        } catch (Exception e) {
            throw new RuntimeException(e.getMessage(), e);
        }
    }

    /**
     * 获取数据升级的具体实现集合
     */
    private Map<String, DataUpgradeService> matchServices(String currentVersion, String appVersion) {
        List<Class<?>> implClasses = ClassHelper.findInterfaceImpls(DataUpgradeService.class, properties.getBasePackage());
        if (implClasses.isEmpty()) {
            return new HashMap<>();
        }

        VersionComparator vc = new VersionComparator();
        Map<String, DataUpgradeService> serviceMaps = new HashMap<>();
        for (Class<?> clazz : implClasses) {
            DataUpgradeVersion annotation = clazz.getAnnotation(DataUpgradeVersion.class);
            if (annotation == null) {
                // 跳过未声明“数据升级版本”注解的实现
                continue;
            }

            String implVersion = annotation.value();
            if (!VersionComparator.isValidVersion(implVersion)) {
                throw new RuntimeException("实现类声明的版本号格式不正确！" + clazz.getSimpleName() + " @DataUpgradeVersion:" + implVersion);
            }

            if (vc.compare(implVersion, currentVersion) > 0 && vc.compare(implVersion, appVersion) <= 0) {
                // log.info("{} {}", clazz.getSimpleName(), implVersion);
                if (serviceMaps.containsKey(implVersion)) {
                    throw new RuntimeException("同个版本数据升级的实现出现重复！@DataUpgradeVersion:" + implVersion);
                }

                try {
                    Constructor<?> constructor = ClassUtils.getConstructorIfAvailable(clazz);
                    if (constructor != null) {
                        serviceMaps.put(implVersion, (DataUpgradeService) constructor.newInstance());
                    }
                } catch (Exception e) {
                    throw new RuntimeException(e);
                }
            }
        }
        return serviceMaps;
    }

    private void updateBootAppInfo(JdbcTemplate jdbcTemplate, String appCode, String appVersion) {
        if (StringUtils.isBlank(appCode)) {
            throw new RuntimeException("appCode的值不能为空！");
        }
        if (StringUtils.isBlank(appVersion)) {
            throw new RuntimeException("appVersion的值不能为空！");
        }
        try {
            jdbcTemplate.update("update boot_app_info set app_version = ? where app_code = ?", appVersion, appCode);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private BootAppInfo queryBootAppInfo(JdbcTemplate jdbcTemplate, String appCode) {
        try {
            List<BootAppInfo> list = jdbcTemplate.query("select app_code,app_version from boot_app_info where app_code = ?",
                    new String[]{appCode}, new BeanPropertyRowMapper<>(BootAppInfo.class));
            return list.isEmpty() ? null : list.get(0);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private void initBootAppInfo(JdbcTemplate jdbcTemplate, String appCode, String appVersion) {
        if (StringUtils.isBlank(appCode)) {
            throw new RuntimeException("appCode的值不能为空！");
        }
        if (StringUtils.isBlank(appVersion)) {
            throw new RuntimeException("appVersion的值不能为空！");
        }
        try {
            String createTableSql = "create table if not exists boot_app_info (" +
                    " app_code varchar(20) not null," +
                    " app_version varchar(10) not null," +
                    " unique key(app_code)" +
                    ")";
            jdbcTemplate.execute(createTableSql);

            jdbcTemplate.update("insert into boot_app_info (app_code,app_version) values(?,?)", appCode, appVersion);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private boolean existBootAppInfo(JdbcTemplate jdbcTemplate) {
        try {
            // 是否存在应用信息表：boot_app_info
            List<String> exists = jdbcTemplate.queryForList("SHOW TABLES LIKE 'boot_app_info'", String.class);
            return !exists.isEmpty();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private HikariDataSource initDataSource() {
        HikariDataSource dataSource = new HikariDataSource();
        dataSource.setPoolName("dataUpgradeDataSource");
        dataSource.setDriverClassName(properties.getDriverClassName());
        dataSource.setJdbcUrl(properties.getDbUrl());
        dataSource.setUsername(properties.getDbUsername());
        dataSource.setPassword(properties.getDbPassword());
        return dataSource;
    }

}
