package com.qmth.boot.data.upgrade.config;

import org.apache.commons.lang3.StringUtils;
import org.springframework.boot.jdbc.DatabaseDriver;
import org.springframework.core.env.ConfigurableEnvironment;

public class DataUpgradeProperties implements DataUpgradeConstants {

    /**
     * 是否启用数据升级处理，默认值为true
     */
    private Boolean enable;

    /**
     * 数据升级服务实现类所在位置的包名，默认扫描包名为：com.qmth
     */
    private String basePackage;

    /**
     * 是否首次安装初始化，默认值为false
     */
    private Boolean firstInstall;

    /**
     * DB用户名
     */
    private String dbUsername;

    /**
     * DB用户密码
     */
    private String dbPassword;

    /**
     * DB链接地址
     */
    private String dbUrl;

    /**
     * DB驱动名称
     */
    private String driverClassName;

    /**
     * 待升级应用码
     */
    private String appCode;

    /**
     * 待升级应用版本号
     */
    private String appVersion;

    public DataUpgradeProperties(ConfigurableEnvironment environment) {
        this.enable = environment.getProperty(ENABLE, Boolean.class, true);
        if (!enable) {
            // 未启用时，不再进行后续配置处理
            return;
        }

        this.basePackage = environment.getProperty(BASE_PACKAGE, String.class, "com.qmth");
        this.firstInstall = environment.getProperty(FIRST_INSTALL, Boolean.class, false);

        this.appCode = environment.getProperty(APP_CODE, String.class);
        if (StringUtils.isBlank(appCode)) {
            throw new RuntimeException(APP_CODE + " 配置值不能为空！");
        }

        this.appVersion = environment.getProperty(APP_VERSION, String.class);
        if (StringUtils.isBlank(appVersion)) {
            throw new RuntimeException(APP_VERSION + " 配置值不能为空！");
        }

        this.dbUsername = environment.getProperty(DB_USERNAME, String.class);
        if (StringUtils.isBlank(dbUsername)) {
            throw new RuntimeException(DB_USERNAME + " 配置值不能为空！");
        }

        this.dbPassword = environment.getProperty(DB_PASSWORD, String.class);
        if (StringUtils.isBlank(dbPassword)) {
            throw new RuntimeException(DB_PASSWORD + " 配置值不能为空！");
        }

        this.dbUrl = environment.getProperty(DB_URL, String.class);
        if (StringUtils.isBlank(dbUrl)) {
            throw new RuntimeException(DB_URL + " 配置值不能为空！");
        }

        String superDbUsername = environment.getProperty(SUPER_DB_USERNAME, String.class);
        if (StringUtils.isNotBlank(superDbUsername)) {
            // 若配置了更高权限的DB账号，则优先使用
            this.dbUsername = superDbUsername;
        }

        String superDbPassword = environment.getProperty(SUPER_DB_PASSWORD, String.class);
        if (StringUtils.isNotBlank(superDbPassword)) {
            // 若配置了更高权限的DB账号，则优先使用
            this.dbPassword = superDbPassword;
        }

        DatabaseDriver databaseDriver = DatabaseDriver.fromJdbcUrl(dbUrl);
        if (DatabaseDriver.UNKNOWN == databaseDriver) {
            throw new RuntimeException(DB_URL + " 配置值有误，不能有效识别出DriverClassName！");
        }
        this.driverClassName = databaseDriver.getDriverClassName();
    }

    public Boolean getEnable() {
        return enable;
    }

    public String getBasePackage() {
        return basePackage;
    }

    public Boolean getFirstInstall() {
        return firstInstall;
    }

    public String getDbUsername() {
        return dbUsername;
    }

    public String getDbPassword() {
        return dbPassword;
    }

    public String getDbUrl() {
        return dbUrl;
    }

    public String getDriverClassName() {
        return driverClassName;
    }

    public String getAppCode() {
        return appCode;
    }

    public String getAppVersion() {
        return appVersion;
    }

}
